@@ -2,6 +2,7 @@ import datetime
 import logging
 from operator import itemgetter
 import itertools
+import collections
 
 import matplotlib as mpl
 import netCDF4 as netcdf
@@ -40,7 +41,7 @@ class BaseLidarMeasurement(object):
         self.info = {}
         self.dimensions = {}
         self.variables = {}
-        self.channels = {}
+        self.channels = collections.OrderedDict()
         self.attributes = {}
         self.files = []
         self.dark_measurement = None
